% This function combines the information from force plates and marker
% informations to acquire a more precise CoM estimate. It further
% re-calculates the ground reaction forces that now fit exactly to the
% calculated CoM movement.
%
% Details of the algorithm can be found in 
% "Combining forces and kinematics for calculating consistent centre of mass trajectories"
% H-M. Maus, A. Seyfarth, S. Grimmer
% The Journal of Experimental Biology 214 (2011), 3511-3517
%
% (c) 2011 Moritz Maus, Lauflabor, University of Jena
%          moritz.maus@uni-jena.de
%
% 
% Usage: 
% [CoM Force] = CoM_combine(CoM, Force, fs, mass [, f_changeover [, sharpness]])
%
%         CoM: initial guess for center of mass trajectory, e.g. from
%              marker set (one component, e.g X) [m]
%       Force: component of the ground reaction force according to CoM [N]
%              ! !  I M P O R T A N T ! !  Force must be offset-free, e.g.
%              components that compensate the gravitation must be removed
%              from the signal. 
%              E.g. For vertical force Fz try Fz = Fz - mean(Fz)
%          fs: sampling frequency [Hz]
%        mass: body weight [Kg]
%     f_changeover: changeover frequency of the algorithm [Hz]
%        sharpness: sharpness of the changeover           [1/Hz]
%
function [c_CoM c_Force] = CoM_combine(mCoM,Force,fs,mass,varargin)
f_changeover  = 1.5;     %[Hz]
sharpness     = 10;      %[1/Hz], changeover-width (1/e) ~ 1/sharpness [Hz]

if nargin > 4
    f_changeover = varargin{1};
    fprintf(' changeover frequency:  %2.3f [Hz]\n',f_changeover);
end %if
    
if nargin > 5
    sharpness = varargin{2};
    fprintf('           sharpness:  %2.3f [1/Hz]\n',sharpness);
end %if


% transpose vectors to column vectors
if size(mCoM,2) > 1
    mCoM = mCoM.';
end %if

if size(Force,2) > 1
    Force = Force.';
end %if

% create frequency vector - a vector that represents the frequencies of the
% measured signal
freq_vec = linspace(0, (length(mCoM)-1)/length(mCoM) * fs, length(mCoM))';           

% calculate weighting vector - a vector that weights the frequencies
wv       = weighting_vector(freq_vec,f_changeover,sharpness)';


% calculate spectra of the velocity signal v.
% v is calculated once by integrating the force, secondly by deriving the
% CoM position

Force = [0; Force] + [Force; 0];
Force = Force(1:end-1)*0.5;
vel_int = (cumsum(Force)); 
                                % + flipud(cumsum(flipud(Force))))/2;
spect_int  = fft(vel_int).'/fs/mass;
spect_diff = fft(gradient(mCoM)).'*fs;

spect_combine = wv.*spect_diff + (1-wv).*spect_int;



% re-calculate the velocity
% use real to convert complex to double; imaginary part is zero.
vz_neu = real(ifft(spect_combine));
         
% calculate consistent CoM and force fata
c_CoM   = cumtrapz(vz_neu)/fs + mCoM(1);
c_Force = gradient(vz_neu)*fs*mass;

function weighting = weighting_vector (freq_vec,f_changeover,sharpness)
% this function creates the weighting vector that weights the components of
% the spectra. It is high for low frequencies, ans low for high
% frequencies (high: <= f_max/2). It is mirrored at f_max/2.
%
% Arguments: 
%           freq_vec     : the vector that contains the frequencies of the
%               signal [Hz]
%           f_changeover : the changeover (1/2) frequency [Hz]
%              sharpness : the width of the changeover, [1/Hz]
%
% The weighting function must be exactly symmetrical except for the first
% argumrnt. So one way to create it that automatically ensures this is:

weighting = 1./( 1 + exp( ( freq_vec(2:end) - f_changeover ) * sharpness ) );
weighting = weighting + flipud(weighting);
weighting = [1; weighting];

